/* LICENSE BLOCK
Part of mde: a Modular D game-oriented Engine
Copyright © 2007-2008 Diggory Hardy

This program is free software; you can redistribute it and/or modify it under the terms of
the GNU General Public License, version 2, as published by the Free Software Foundation.

This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License along
with this program; if not, write to the Free Software Foundation, Inc.,
51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA. */

/// Contains the base class for all mde exceptions plus some exception classes.
module mde.exception;

/** Base class for all mde Exceptions.
 *
 * All packages should have their own base exception type extending this one, and for each package
 * level a CTOR taking a message should pass the message to the super.
 * A CTOR not taking a message and calling the super without a parameter may also be provided.
 *
 * The static string symbol
 * should be overriden as below so that it ends up as something like "mde.file" or
 * "mde.pkg.file.Class" describing where the exception was thrown. (Since only methods overload
 * correctly, symbol is made static and an overloadable method is used to access the correct symbol.)
 */
class mdeException : Exception {
    static const symbol = "mde";	/// Override in derived classes to name the module where the error occured.
    char[] getSymbol () {
        return symbol;
    }
    this (char[] msg) {
        super(getSymbol() ~ ": " ~ msg);
    }
    this () {			// No supplied error message.
        super(symbol);
    }
}

/// Thrown when Init fails.
class InitException : mdeException {
    // NOTE: if symbol is final, it can't be modified in the static this(), but as const it can
    static const char[] symbol;
    static this () {	symbol = super.symbol ~ ".Init";	}
    char[] getSymbol () {
        return symbol;
    }
    
    this (char[] msg) {
        super(msg);
    }
}

/* LICENSE BLOCK
Part of mde: a Modular D game-oriented Engine
Copyright © 2007-2008 Diggory Hardy

This program is free software; you can redistribute it and/or modify it under the terms of
the GNU General Public License, version 2, as published by the Free Software Foundation.

This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License along
with this program; if not, write to the Free Software Foundation, Inc.,
51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA. */

/// Thrown when loading options fails.
class optionsLoadException : mdeException {
    // NOTE: if symbol is final, it can't be modified in the static this(), but as const it can
    static const char[] symbol;
    static this () {	symbol = super.symbol ~ ".options";	}
    char[] getSymbol () {
        return symbol;
    }
    
    this (char[] msg) {
        super(msg);
    }
}

/// Thrown when loading strings for the requested name and current locale fails.
class L10nLoadException : mdeException {
    // NOTE: if symbol is final, it can't be modified in the static this(), but as const it can
    static const char[] symbol;
    static this () {	symbol = super.symbol ~ ".i18n.I18nTranslation";	}
    char[] getSymbol () {
        return symbol;
    }
    
    this (char[] msg) {
        super(msg);
    }
}

debug (mdeUnitTest) {
    import tango.util.log.Log : Log, Logger;

    private Logger logger;
    static this() {
        logger = Log.getLogger ("mde.exception");
    }
    
    unittest {
        // Check message prepending works correctly.
        mdeException mE = new InitException("");
        assert (mE.getSymbol() == "mde.Init", mE.getSymbol());
        try {
            throw new InitException ("ABC");
            assert (false);
        } catch (Exception e) {
            assert (e.msg == "mde.Init: ABC", e.msg);
        }
    
        logger.info ("Unittest complete.");
    }
}
